#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define PIN_BLINK 0
#define PIN_RESET 1
#define PIN_CLOCK 2
#define PIN_POWER 3
#define PIN_SENSE 4

#define BIT_BLINK (1<<PIN_BLINK)
#define BIT_RESET (1<<PIN_RESET)
#define BIT_CLOCK (1<<PIN_CLOCK)
#define BIT_POWER (1<<PIN_POWER)
#define BIT_SENSE (1<<PIN_SENSE)

#define BITS_OUT (BIT_BLINK|BIT_RESET|BIT_CLOCK|BIT_POWER)
#define BITS_IN ~BITS_OUT

#define MIN_POWER_LEVEL 0
#define MAX_POWER_LEVEL 255

#define DISPLAY_PERIOD 1000
//#define DISPLAY_PERIOD 238
#define MAINS_PERIOD 48
#define MAINS_POWER_CONST (MAINS_PERIOD*MAX_POWER_LEVEL)

#define ADC_AVG_LEVEL 5

volatile unsigned short _timer = 0;
volatile unsigned long _millis = 0;

int power_level = 0;
unsigned short power_state = 0;
unsigned long power_time = 0;
unsigned long display_time = 0;

void start_timer(void)
{
 // PWM Settings
 TCCR0A = ((1<<WGM01)|(1<<WGM00)); // non inverting fast pwm, OC0A + OC0B pins
 // prescaler
 TCCR0B = (1<<CS00); // 1, 1MHz
 //TCCR0B = (1<<CS01); // 8, (1/8)MHz=125kHz
 //TCCR0B = ((1<<CS01)|(1<<CS00)); // 64, (1/64)MHz=15.625kHz
 //TCCR0B = (1<<CS02); // 256, (1/256)MHz=3.90625kHz
 //TCCR0B = ((1<<CS02)|(1<<CS00)); // 1024, (1/1024)MHz=976.5625Hz
 // enable overflow interrupt (TOIE2)
 TIMSK0 = (1<<OCIE0A);
 OCR0A = 0;
 OCR0B = 0;
 asm("sei");
}

ISR(TIM0_COMPA_vect)
{
 //PORTB |= BIT_BLINK;
 unsigned short blink = PORTB&BIT_BLINK;
 if (blink)
 {
  PORTB &= ~BIT_BLINK;
 }
 else
 {
  PORTB |= BIT_BLINK;
 }
 _timer++;
 //_millis++;
 //PORTB &= ~BIT_BLINK;
}


int main(void)
{
 // initialize io
 DDRB |= BITS_OUT;
 PORTB &= ~BITS_OUT;
 // begin reset display
 PORTB |= BIT_RESET;
 // enable ADC, prescaler to clk/8
 ADCSRA |= (1<<ADEN)|(1<<ADPS1)|(1<<ADPS0);
 ADMUX |= (1<<ADLAR)|(1<<MUX1);
 //ADMUX |= (1<<MUX1);
 // end reset display
 PORTB &= ~BIT_RESET;
 //
 start_timer();
 //
 while(1)
 {
  unsigned long new_time = _timer;
  if (new_time>power_time)
  {
   // read sensor
   ADCSRA |= (1<<ADEN)|(1<<ADSC);
   while (ADCSRA & (1<<ADSC));
   ADCSRA &= ~(1<<ADEN);
   int new_power_level = ADCH;
   if (((new_time-display_time)>DISPLAY_PERIOD)&&
       ((MIN_POWER_LEVEL==new_power_level)||
       (MAX_POWER_LEVEL==new_power_level)||
       (power_level!=new_power_level)))
   //if ((new_time-display_time)>DISPLAY_PERIOD)
   {
    unsigned short display_value = (unsigned short)((power_level*25)/64);
    // reset display
    PORTB |= BIT_RESET;
    PORTB &= ~BIT_RESET;
    // clock display up to value
    //int display_value = new_power_level;
    //while (display_value--)
    while (display_value--)
    {
     PORTB |= BIT_CLOCK;
     PORTB &= ~BIT_CLOCK;
    }
    //_delay_ms(100);
    display_time = new_time;
    power_level = (power_level+new_power_level)/2;
   }
   //power_level = (power_level*31+new_power_level)/32;
   //
   unsigned short new_power_state = power_state;
   if (power_level>=MAX_POWER_LEVEL)
   {
    new_power_state = 1;
    //_delay_ms(100);
   }
   else if (power_level<=MIN_POWER_LEVEL)
   {
    new_power_state = 0;
    //_delay_ms(100);
   }
   else
   {
    int wait_time = new_time - power_time;
    if (power_state)
    {
     if (wait_time*(MAX_POWER_LEVEL-power_level) > MAINS_POWER_CONST)
     {
      new_power_state = 0;
      power_time = new_time;
     }
    }
    else
    {
     if (wait_time*power_level > MAINS_POWER_CONST)
     {
      new_power_state = 1;
      power_time = new_time;
     }
    }
   }
   power_state = new_power_state;
   if (power_state)
   {
    PORTB |= BIT_POWER;
    //PORTB |= (BIT_POWER|BIT_BLINK);
   }
   else
   {
    PORTB &= ~BIT_POWER;
    //PORTB &= ~(BIT_POWER|BIT_BLINK);
   }
   //PORTB = (PORTB&(~BIT_POWER))|(power_state<<PIN_POWER);
  }
  else
  {
   power_time = new_time;
   display_time = new_time;
  }
  /*
  // blink all pins
  PORTB |= 0x13;//BIT_POWER;
  _delay_ms(100);
  PORTB &= ~0x13;//~BIT_POWER;
  _delay_ms(100);
  */
 };
 return 0;
}
